@@ -1,14 +1,14 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.34';
+our $VERSION = '0.35';
 
 use Exporter 'import';
 our @EXPORT_OK = qw(
     generate portrait landscape
     north east south west
     layout checkerboard animate
-    chess checkers conway
+    chess checkers conway sudoku
     calculator dk shroom
     calendar banner maze
 );
@@ -32,6 +32,7 @@ sub conway          { Spreadsheet::HTML::Presets::conway(           @_ ) }
 sub calculator      { Spreadsheet::HTML::Presets::calculator(       @_ ) }
 sub chess           { Spreadsheet::HTML::Presets::chess(            @_ ) }
 sub checkers        { Spreadsheet::HTML::Presets::checkers(         @_ ) }
+sub sudoku          { Spreadsheet::HTML::Presets::sudoku(           @_ ) }
 sub checkerboard    { Spreadsheet::HTML::Presets::checkerboard(     @_ ) }
 sub calendar        { Spreadsheet::HTML::Presets::calendar(         @_ ) }
 sub animate         { Spreadsheet::HTML::Presets::animate(          @_ ) }
@@ -182,19 +183,31 @@ sub _make_table {
 
     if ($args{tgroups}) {
 
-        my ($head, @body) = @{ $args{data} };
-        my $foot = pop @body if $args{tgroups} > 1 and scalar @{ $args{data} } > 2;
+        my @body = @{ $args{data} };
+        my $head = shift @body unless $args{matrix} and scalar @{ $args{data} } > 2;
+        my $foot = pop @body if !$args{matrix} and $args{tgroups} > 1 and scalar @{ $args{data} } > 2;
 
-        my $head_row  =     { tag => 'tr', attr => $args{tr}, cdata => $head };
-        my $foot_row  =     { tag => 'tr', attr => $args{tr}, cdata => $foot };
-        my @body_rows = map { tag => 'tr', attr => $args{tr}, cdata => $_ }, @body;
+        my $head_row  = { tag => 'tr', attr => $args{tr}, cdata => $head };
+        my $foot_row  = { tag => 'tr', attr => $args{tr}, cdata => $foot };
+        my $body_rows = [ map { tag => 'tr', attr => $args{tr}, cdata => $_ }, @body ];
+
+        if (int($args{group} || 0) > 1) {
+            $body_rows = [
+                map [ @$body_rows[$_ .. $_ + $args{group} - 1] ],
+                _range( 0, $#$body_rows, $args{group} )
+            ];
+            pop @{ $body_rows->[-1] } while !defined $body_rows->[-1][-1];
+        } else {
+            $body_rows = [ $body_rows ];
+        }
 
         push @cdata, (
-            { tag => 'thead', attr => $args{thead}, cdata => $head_row },
+            ( $head ? { tag => 'thead', attr => $args{thead}, cdata => $head_row } : () ),
             ( $foot ? { tag => 'tfoot', attr => $args{tfoot}, cdata => $foot_row } : () ),
-            { tag => 'tbody', attr => $args{tbody}, cdata => [@body_rows] }
+            ( map { tag => 'tbody', attr => $args{tbody}, cdata => [ @$_ ] }, @$body_rows ),
         );
 
+
     } else {
         push @cdata, map { tag => 'tr', attr => $args{tr}, cdata => $_ }, @{ $args{data} };
     }
@@ -228,16 +241,24 @@ sub _colgroup {
 
     if (ref($args{col}) eq 'ARRAY') {
 
-        @colgroup = ({
-            tag   => 'colgroup',
-            attr  => $args{colgroup},
-            cdata => [ map { tag => 'col', attr => $_ }, @{ $args{col} } ]
-        }); 
+        if (ref $args{colgroup} eq 'ARRAY') {
+            @colgroup = map {
+                tag   => 'colgroup',
+                attr  => $_,
+                cdata => [ map { tag => 'col', attr => $_ }, @{ $args{col} } ]
+            }, @{ $args{colgroup} }; 
+        } else {
+            @colgroup = {
+                tag   => 'colgroup',
+                attr  => $args{colgroup},
+                cdata => [ map { tag => 'col', attr => $_ }, @{ $args{col} } ]
+            }; 
+        }
 
     } else {
 
         $args{colgroup} = [ $args{colgroup} ] if ref($args{colgroup}) eq 'HASH';
-        if (ref($args{colgroup}) eq 'ARRAY') {
+        if (ref $args{colgroup} eq 'ARRAY') {
             @colgroup = map { tag => 'colgroup', attr => $_ }, @{ $args{colgroup} };
         }
     }
@@ -274,7 +295,7 @@ sub _args {
         $data = $self->{data} unless $data or $args->{file};
     }
 
-    $data = Spreadsheet::HTML::File::Loader::parse( $args->{file} ) if $args->{file};
+    $data = Spreadsheet::HTML::File::Loader::parse( $args ) if $args->{file};
     $data = [ $data ] unless ref($data);
     $data = [ $data ] unless ref($data->[0]);
 
@@ -317,19 +338,22 @@ Spreadsheet::HTML - Just another HTML table generator.
 
     $generator = Spreadsheet::HTML->new( data => $data, indent => "\t" );
     print $generator->portrait;
-    print $generator->landscape;
+    print $generator->landscape( encodes => '<>' );
 
     # load from files (first table found)
     $generator = Spreadsheet::HTML->new( file => 'data.xls', cache => 1 );
+    print $generator->generate( preserve => 1 );
+
+Procedural interface:
 
-    # non OO
     use Spreadsheet::HTML qw( portrait landscape );
+
     print portrait( $data, td => sub { sprintf "%02d", shift } );
     print landscape( $data, tr => { class => [qw(odd even)] } );
 
 =head1 DESCRIPTION
 
-Generate HTML4, XHTML and HTML5 tables with ease. Provides a handful
+Generate HTML tables with ease (HTML4, XHTML and HTML5). Provides a handful
 of distinctly named methods to control overall table orientation.
 These methods in turn accept a number of distinctly named attributes
 for directing what tags and attributes to use.
@@ -346,7 +370,7 @@ exception of C<new>, all methods return an HTML table as a scalar string.
 
 =item * C<new( %params )>
 
-  my $table = Spreadsheet::HTML->new( data => $data );
+  my $generator = Spreadsheet::HTML->new( data => $data );
 
 Constructs object. Accepts the same named parameters as the table
 generating methods below:
@@ -357,91 +381,48 @@ generating methods below:
 
 =item * C<north( %params )>
 
+These three methods all generate an HTML table with its headings positioned at the top.
+
 =for html
 <table style="border: 1px dashed #A0A0A0"><tr><td><b>&nbsp;&nbsp;heading1&nbsp;&nbsp;</b></td><td><b>&nbsp;&nbsp;heading2&nbsp;&nbsp;</b></td><td><b>&nbsp;&nbsp;heading3&nbsp;&nbsp;</b></td><td><b>&nbsp;&nbsp;heading4&nbsp;&nbsp;</b></td></tr><tr><td>&nbsp;&nbsp;row1col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row1col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row1col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row1col4&nbsp;&nbsp;</td></tr><tr><td>&nbsp;&nbsp;row2col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col4&nbsp;&nbsp;</td></tr><tr><td>&nbsp;&nbsp;row3col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col4&nbsp;&nbsp;</td></tr></table>
 
-  $html = $table->generate( table => {border => 1}, encode => '<>' );
-  print Spreadsheet::HTML::generate( data => $data, indent => "\t" );
-
-Headers on top. C<north()> is an alias for C<portrait()> which in turn
-calls C<generate> like so:
-
-  generate( theta => 0, %params )
+  print $generator->generate();
+  print $generator->portrait();
+  print $generator->north();
 
 =item * C<landscape( %params )>
 
 =item * C<west( %params )>
 
+These two methods generate an HTML table with its headings positioned at the left.
+
 =for html
 <table style="border: 1px dashed #A0A0A0"><tr><td><b>&nbsp;&nbsp;heading1&nbsp;&nbsp;</b></td><td>&nbsp;&nbsp;row1col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col1&nbsp;&nbsp;</td></tr><tr><td><b>&nbsp;&nbsp;heading2&nbsp;&nbsp;</b></td><td>&nbsp;&nbsp;row1col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col2&nbsp;&nbsp;</td></tr><tr><td><b>&nbsp;&nbsp;heading3&nbsp;&nbsp;</b></td><td>&nbsp;&nbsp;row1col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col3&nbsp;&nbsp;</td></tr><tr><td><b>&nbsp;&nbsp;heading4&nbsp;&nbsp;</b></td><td>&nbsp;&nbsp;row1col4&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col4&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col4&nbsp;&nbsp;</td></tr></table>
 
-Headers on left. C<west()> is an alias for C<landscape()> which
-in turn calls C<generate> like so:
-
-  generate( theta => -270 )
+  print $generator->landscape();
+  print $generator->west();
 
 =item * C<south( %params )>
 
+This method generates an HTML table with its headings positioned at the bottom.
+
 =for html
 <table style="border: 1px dashed #A0A0A0"><tr><td>&nbsp;&nbsp;row1col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row1col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row1col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row1col4&nbsp;&nbsp;</td></tr><tr><td>&nbsp;&nbsp;row2col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col4&nbsp;&nbsp;</td></tr><tr><td>&nbsp;&nbsp;row3col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col4&nbsp;&nbsp;</td></tr><tr><td><b>&nbsp;&nbsp;heading1&nbsp;&nbsp;</b></td><td><b>&nbsp;&nbsp;heading2&nbsp;&nbsp;</b></td><td><b>&nbsp;&nbsp;heading3&nbsp;&nbsp;</b></td><td><b>&nbsp;&nbsp;heading4&nbsp;&nbsp;</b></td></tr></table>
 
-Headers on bottom. Same as
-
-  generate( theta => -180, pinhead => 1 )
+  print $generator->south();
 
 =item * C<east( %params )>
 
+This method generates an HTML table with its headings positioned at the right.
+
 =for html
 <table style="border: 1px dashed #A0A0A0"><tr><td>&nbsp;&nbsp;row1col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col1&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col1&nbsp;&nbsp;</td><td><b>&nbsp;&nbsp;heading1&nbsp;&nbsp;</b></td></tr><tr><td>&nbsp;&nbsp;row1col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col2&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col2&nbsp;&nbsp;</td><td><b>&nbsp;&nbsp;heading2&nbsp;&nbsp;</b></td></tr><tr><td>&nbsp;&nbsp;row1col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col3&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col3&nbsp;&nbsp;</td><td><b>&nbsp;&nbsp;heading3&nbsp;&nbsp;</b></td></tr><tr><td>&nbsp;&nbsp;row1col4&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row2col4&nbsp;&nbsp;</td><td>&nbsp;&nbsp;row3col4&nbsp;&nbsp;</td><td><b>&nbsp;&nbsp;heading4&nbsp;&nbsp;</b></td></tr></table>
 
-Headers on right. Same as
-
-  generate( theta => 90, pinhead => 1 )
+  print $generator->east();
 
 =back
 
-Because these methods are all essentially aliases for C<generate()>
-(with C<theta> being preset for you), you can override their behavior by
-calling C<generate()> with any configuration of parameters that you like.
-
-For most cases, C<portrait()> and C<landscape()> are all you need.
-Everything else is C<bells_and_whistles>.
-
-=head2 PRESETS
-
-The following presets are availble for creating tables that can be used
-with little to no additional coding.
-
-=over 4
-
-=item * C<layout( %params )>
-
-=item * C<conway( on, off, fade, jquery, %params )>
-
-=item * C<calculator( jquery, %params )>
-
-=item * C<calendar( month, year, %params )>
-
-=item * C<checkerboard( colors, %params )>
-
-=item * C<animate( fgdirection, bgdirection, jquery, %params )>
-
-=item * C<banner( on, off, text, font, %params )>
-
-=item * C<maze( %params )>
-
-=item * C<chess( %params )>
-
-=item * C<checkers( %params )>
-
-=item * C<dk( %params )>
-
-=item * C<shroom( %params )>
-
-=back
-
-See L<Spreadsheet::HTML::Presets> for more documentation (and the source
-for more usage examples).
+For most cases, C<portrait> and C<landscape> are all you need.
 
 =head1 PARAMETERS
 
@@ -471,6 +452,24 @@ are XLS, CSV, JSON, YAML and HTML (first table found).
 
   file => 'foo.json'
 
+If you want to filter your database queries then install
+L<DBIx::HTML> which uses this module as its generator.
+
+  use DBIx::HTML;
+  my $generator = DBIx::HTML->connect( @db_credentials );
+  $generator->do( $query, @bind_args );
+  print $table->portrait( %params );
+
+=item * C<preserve>
+
+Can be supplied in addition to C<file>. Attempts to copy
+over all formatting styles from original document to table.
+Styles are not currently deduped, so use with care as the
+final output will contain a lot of redundant cell styles.
+Work is being done to alleviate this.
+
+  file => 'foo.xls', preserve => 1
+
 =item * C<fill>
 
 Can be supplied instead of C<data> to generate empty
@@ -552,6 +551,15 @@ of the <tbody> section instead. (loose)
 When C<tgroups> is set to 2, the <tfoot> section is found
 in between the <thead> and <tbody> sections. (strict)
 
+=item * C<group>
+
+Will chunk body rows into tbody groups of size C<group>.
+
+  group => 4
+
+Currently only accepts integers although it should be possilbe
+to group by changes in a column in the future.
+
 =item * C<cache: 0 or 1>
 
 Preserve data after it has been processed (and loaded).
@@ -600,7 +608,7 @@ leading dashes to seperate them from literal and tag parameters.
 
 =item * C<-rX>
 
-Apply this callback subroutine to the entire row X.
+Apply this callback subroutine to all cells in row X.
 (0 index based)
 
   -r3 => sub { uc shift }
@@ -615,7 +623,7 @@ Or both:
 
 =item * C<-cX>
 
-Apply this callback to the entire column X.
+Apply this callback to all cells in column X.
 (0 index based)
 
   -c4 => sub { sprintf "%02d", shift || 0 }
@@ -649,7 +657,7 @@ to the cell at row X and column X. (0 index based)
 =head2 TAG PARAMETERS
 
 Tag parameters provide a means to control the attributes
-of the table's tags, and in the case of <td> and <tr> the
+of the table's tags, and in the case of <th> and <td> the
 contents via callback subroutines. Although similar in form,
 they are differentiated from litertal parameters because they
 share the names of the actual HTML table tags.
@@ -706,8 +714,50 @@ Add colgroup tag(s) to the table. Use an AoH for multiple.
 Add col tag(s) to the table. Use an AoH for multiple. Wraps
 tags within a colgroup tag. Same usage as C<colgroup>.
 
+  col => { span => 2, style => { 'background-color' => 'orange' } }
+
+  col => [ { span => 20 }, { span => 1, class => 'end' } ]
+
 =back
 
+=head2 PRESETS
+
+The following presets are availble for creating tables that can be used
+with little to no additional coding.
+
+=over 4
+
+=item * C<layout( %params )>
+
+=item * C<conway( on, off, fade, jquery, %params )>
+
+=item * C<calculator( jquery, %params )>
+
+=item * C<calendar( month, year, %params )>
+
+=item * C<checkerboard( colors, %params )>
+
+=item * C<animate( fgdirection, bgdirection, jquery, %params )>
+
+=item * C<banner( on, off, text, font, %params )>
+
+=item * C<maze( on, off, fill, %params )>
+
+=item * C<sudoku( blanks, attempts, %params )>
+
+=item * C<chess( %params )>
+
+=item * C<checkers( %params )>
+
+=item * C<dk( %params )>
+
+=item * C<shroom( %params )>
+
+=back
+
+See L<Spreadsheet::HTML::Presets> for more documentation (and the source
+for more usage examples).
+
 =head1 REQUIRES
 
 =over 4
@@ -783,15 +833,6 @@ value contol, rotating attributes and totals/subtotals.
 
 =head1 BUGS AND LIMITATIONS
 
-Support for <col> and <colgroup> has not been adequately tested
-as i honestly do not fully understand why two tags exist when
-one should do the trick. If you cannot achieve the behavior you
-desire from this module's generation of <col> and <colgroup>
-tags please feel free to submit a detailed bug report. The same
-goes for colspan and rowspan attributes -- very little testing
-has been done because this module can limit its problem domain
-to grid like tables of equal sized cells. But if there's a way ...
-
 Please report any bugs or feature requests to either
 
 =over 4
