@@ -4,8 +4,10 @@ use warnings FATAL => 'all';
 
 use Spreadsheet::HTML;
 use Spreadsheet::HTML::Presets::Animate;
-use Spreadsheet::HTML::Presets::Conway;
 use Spreadsheet::HTML::Presets::Calculator;
+use Spreadsheet::HTML::Presets::Conway;
+use Spreadsheet::HTML::Presets::Chess;
+use Spreadsheet::HTML::Presets::Sudoku;
 
 eval "use Color::Spectrum";
 our $NO_SPECTRUM = $@;
@@ -17,6 +19,8 @@ eval "use Time::Piece";
 our $NO_TIMEPIECE = $@;
 eval "use List::Util";
 our $NO_LISTUTIL = $@;
+eval "use Games::Sudoku::Component";
+our $NO_SUDOKU = $@;
 
 sub layout {
     my ($self,$data,$args);
@@ -53,7 +57,7 @@ sub checkerboard {
         headings => sub { join(' ', map { sprintf '<b>%s</b>', ucfirst(lc($_)) } split ('_', shift || '')) },
         @_,
         wrap => 0,
-        td => { %{ $args->{td} || {} }, style  => { 'background-color' => $colors } },
+        td => { %{ $args->{td} || {} }, style  => { %{ $args->{td}{style} || {} }, 'background-color' => $colors } },
     );
 
     $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
@@ -86,8 +90,6 @@ sub animate {
     );
 
     my $js = Spreadsheet::HTML::Presets::Animate::_javascript( %$args );
-    delete $args->{$_} for qw( fgdirection bgdirection fx fy bx by interval jquery );
-
     return( $js, @args ) if $args->{animate};
 
     my $table = $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
@@ -147,8 +149,68 @@ sub banner {
     return $table;
 }
 
+sub sudoku {
+    my ($self,$data,$args);
+    $self = shift if ref($_[0]) =~ /^Spreadsheet::HTML/;
+    ($self,$data,$args) = $self ? $self->_args( @_ ) : Spreadsheet::HTML::_args( @_ );
+    $args->{attempts}   = defined $args->{attempts} ? int($args->{attempts} || 0) : 4;
+    $args->{blanks}     = int($args->{blanks} || 0) || 50;
+    $args->{size}       = 9; # Games::Sudoku::Component only accepts perfect squares and only 9 is fast
+
+    my @cells;
+    my ($solved,$unsolved) = ('','');
+    unless ($NO_SUDOKU) {
+        my $board = Games::Sudoku::Component->new( size => $args->{size} );
+        while (!$board->is_solved and $args->{attempts}-- > 0) {
+            $board->generate( blanks => $args->{blanks} );
+            $unsolved = $board->as_string;
+            $board->solve;
+        }
+
+        my %td_attr = ( style => { border => 'solid thin', 'text-align' => 'center', padding => '.5em', 'font-family' => 'Lucida Grande' } );
+        my $auto = HTML::AutoTag->new;
+        my %input_attr = ( class => 'sudoku', size=> 1, style => { 'text-align' => 'center', border => '0px', 'font-size' => 'medium',  color => 'red' } );
+
+        if ($board->is_solved) {
+            $solved = $board->as_string;
+            my @lines = split /\n/, $unsolved;
+            for my $row (0 .. $#lines) {
+                my @chars = split /\s/, $lines[$row];
+                for my $col (0 .. $#chars) {
+                    my $id  = "${row}-${col}";
+                    my $sub = $chars[$col] ? sub { $chars[$col] } : sub { $auto->tag( tag => 'input', attr => { %input_attr, id => "input-$id" } ) };
+                    push @cells, ( "-r${row}c${col}" => [ { %td_attr, id => "td-$id" }, $sub ] );
+                }
+            }
+        }
+    }
+
+    my $sqrt = int(sqrt( $args->{size} ));
+    my @args = (
+        @_,
+        @cells,
+        table    => { id => 'sudoku', style => { 'border-collapse' => 'collapse' } },
+        tbody    => { style => { border => 'solid medium' } },
+        tr       => { id => [ map "sudoku-$_", 0 .. $sqrt - 1] },
+        colgroup => [ ({ style => { border => 'solid medium' } }) x $sqrt ],
+        col      => [ ({}) x $sqrt ],
+        data     => [],
+        fill     => sprintf( '%sx%s', ($args->{size}) x 2 ),
+        wrap     => 0,
+        tgroups  => 1,
+        group    => $sqrt,
+        matrix   => 1,
+        headless => 0,
+        theta    => 0,
+        animate  => 0,
+    );
+
+    my $js    = Spreadsheet::HTML::Presets::Sudoku::_javascript( %$args );
+    my $table = $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
+    return $js . $table;
+}
+
 sub maze {
-    no warnings;
     my ($self,$data,$args);
     $self = shift if ref($_[0]) =~ /^Spreadsheet::HTML/;
     ($self,$data,$args) = $self ? $self->_args( @_ ) : Spreadsheet::HTML::_args( @_ );
@@ -181,7 +243,7 @@ sub maze {
                 [ 2, $grid[ $curr->{y} ][ $curr->{x} + 1 ] ], # east
                 [ 1, $grid[ $curr->{y} + 1 ][ $curr->{x} ] ], # south
                 [ 0, $grid[ $curr->{y} ][ $curr->{x} - 1 ] ], # west
-            ) { push @neighbors, $_ if List::Util::sum( @{ $_->[1]->{walls} } ) == 4 }
+            ) { no warnings; push @neighbors, $_ if List::Util::sum( @{ $_->[1]->{walls} } ) == 4 }
 
             if (@neighbors) {
                 my ($pos,$cell) = @{ $neighbors[rand @neighbors] };
@@ -323,9 +385,7 @@ sub conway {
         @_,
     );
 
-    my $js = Spreadsheet::HTML::Presets::Conway::_javascript( %$args );
-    delete $args->{$_} for qw( jquery off on colors fade interval );
-
+    my $js    = Spreadsheet::HTML::Presets::Conway::_javascript( %$args );
     my $table = $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
     return $js . $table;
 }
@@ -389,36 +449,44 @@ sub calendar {
 }
 
 sub checkers {
-    my $self = shift if ref($_[0]) =~ /^Spreadsheet::HTML/;
+    my ($self,$data,$args);
+    $self = shift if ref($_[0]) =~ /^Spreadsheet::HTML/;
+    ($self,$data,$args) = $self ? $self->_args( @_ ) : Spreadsheet::HTML::_args( @_ );
 
     my @data = (
-        [ '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;' ],
-        [ '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;', '' ],
-        [ '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;' ],
-        [], [],
-        [ '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '' ],
-        [ '', '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;' ],
-        [ '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '' ],
+        [ '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;' ],
+        [ '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;', '' ],
+        [ '', '&#9922;', '', '&#9922;', '', '&#9922;', '', '&#9922;' ],
+        [ ('') x 8 ], [ ('') x 8 ],
+        [ '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '' ],
+        [ '', '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;' ],
+        [ '&#9920;', '', '&#9920;', '', '&#9920;', '', '&#9920;', '' ],
     );
 
+    my $sub = sub { sprintf '<div class="game-piece">%s</div>', shift };
+
     my @args = (
         table => {
             width => '65%',
             style => {
                 border => 'thick outset',
+                %{ $args->{table}{style} || {} },
             },
-        },
-        td => {
-            height => 65,
-            width  => 65,
-            align  => 'center',
-            style  => { 
-                'font-size' => 'xx-large',
-                border => 'thin inset',
-                'background-color' => [ ('white', 'red')x4, ('red', 'white')x4 ]
-            }
+            %{ $args->{table} || {} },
         },
         @_,
+        td => [
+            {
+                height => 65,
+                width  => 65,
+                align  => 'center',
+                style  => { 
+                    'font-size' => 'xx-large',
+                    border => 'thin inset',
+                    'background-color' => [ ('white', 'red')x4, ('red', 'white')x4 ]
+                }
+            }, sub { $_[0] ? qq(<div class="game-piece">$_[0]</div>) : '' }
+        ],
         tgroups  => 0,
         headless => 0,
         pinhead  => 0,
@@ -428,16 +496,20 @@ sub checkers {
         data     => \@data,
     );
 
-    $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
+    my $js    = Spreadsheet::HTML::Presets::Chess::_javascript( %$args );
+    my $table = $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
+    return $js . $table;
 }
 
 sub chess {
-    my $self = shift if ref($_[0]) =~ /^Spreadsheet::HTML/;
+    my ($self,$data,$args);
+    $self = shift if ref($_[0]) =~ /^Spreadsheet::HTML/;
+    ($self,$data,$args) = $self ? $self->_args( @_ ) : Spreadsheet::HTML::_args( @_ );
 
     my @data = (
         [ '&#9820;', '&#9822;', '&#9821;', '&#9819;', '&#9818;', '&#9821;', '&#9822;', '&#9820;' ],
         [ '&#9823;', '&#9823;', '&#9823;', '&#9823;', '&#9823;', '&#9823;', '&#9823;', '&#9823;' ],
-        [], [], [], [],
+        [ ('') x 8 ], [ ('') x 8 ], [ ('') x 8 ], [ ('') x 8 ],
         [ '&#9817;', '&#9817;', '&#9817;', '&#9817;', '&#9817;', '&#9817;', '&#9817;', '&#9817;' ],
         [ '&#9814;', '&#9816;', '&#9815;', '&#9813;', '&#9812;', '&#9815;', '&#9816;', '&#9814;' ],
     );
@@ -447,19 +519,23 @@ sub chess {
             width => '65%',
             style => {
                 border => 'thick outset',
+                %{ $args->{table}{style} || {} },
             },
-        },
-        td => {
-            height => 65,
-            width  => 65,
-            align  => 'center',
-            style  => { 
-                'font-size' => 'xx-large',
-                border => 'thin inset',
-                'background-color' => [ ('white', '#aaaaaa')x4, ('#aaaaaa', 'white')x4 ]
-            }
+            %{ $args->{table} || {} },
         },
         @_,
+        td => [
+            {
+                height => 65,
+                width  => 65,
+                align  => 'center',
+                style  => { 
+                    'font-size' => 'xx-large',
+                    border => 'thin inset',
+                    'background-color' => [ ('white', '#aaaaaa')x4, ('#aaaaaa', 'white')x4 ]
+                }
+            }, sub { $_[0] ? qq(<div class="game-piece">$_[0]</div>) : '' }
+        ],
         tgroups  => 0,
         headless => 0,
         pinhead  => 0,
@@ -469,7 +545,9 @@ sub chess {
         data     => \@data,
     );
 
-    $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
+    my $js    = Spreadsheet::HTML::Presets::Chess::_javascript( %$args );
+    my $table = $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
+    return $js . $table;
 }
 
 sub dk {
@@ -616,11 +694,9 @@ my $tmpl = '
     $self ? $self->generate( @args ) : Spreadsheet::HTML::generate( @args );
 }
 
-sub _html_tmpl {
+sub _js_wrapper {
     my %args = @_;
 
-    $args{jquery} ||= 'https://ajax.googleapis.com/ajax/libs/jquery/2.1.1/jquery.min.js';
-
     unless ($NO_MINIFY) {
         $args{code} = JavaScript::Minifier::minify(
             input      => $args{code},
@@ -629,12 +705,13 @@ sub _html_tmpl {
         );
     }
 
-    my $tmpl = <<'END_HTML';
-<script src="%s"></script>
-<script type="text/javascript">%s</script>
-END_HTML
+    $args{jquery} ||= 'https://ajax.googleapis.com/ajax/libs/jquery/2.1.4/jquery.min.js';
 
-    return sprintf $tmpl, $args{jquery}, $args{code};
+    my $html = sprintf qq{<script type="text/javascript" src="%s"></script>\n}, $args{jquery};
+    $html   .= sprintf qq{<script type="text/javascript" src="%s"></script>\n}, $args{jqueryui} if $args{jqueryui};
+    $html   .= sprintf qq{<script type="text/javascript">%s</script>\n}, $args{code};
+
+    return $html;
 }
 
 =head1 NAME
@@ -757,6 +834,20 @@ Generates a static maze.
 
   maze( fill => '10x10', on => 'red', off => 'black' ) 
 
+=item * C<sudoku( blanks, attempts, %params )>
+
+Generates a static unsolved 9x9 sudoku board. You must have
+L<Games::Sudoku::Component> installed, which currently
+has no dependencies and is very fast and reliable. You can
+specify how many cells to leave unsolved with C<blanks>.
+
+  sudoku( blanks => 50 ) 
+
+Four attempts are made to find a solveable board, you can
+override with C<attempts>.
+
+  sudoku( attempts => 1 ) 
+
 =item * C<checkers( %params )>
 
 Generates a static checkers game board (US).
@@ -771,57 +862,6 @@ Generates a static chess game board.
 
 =back
 
-=head1 CUSTOMIZATION
-
-You can use the methods in this package as an example for
-constructing your own custom Spreadsheet::HTML generators.
-
-  use Spreadsheet::HTML;
-
-  sub my_generator {
-      my ($self,$data,$params);
-      $self = shift if ref($_[0]) =~ /^Spreadsheet::HTML/;
-      ($self,$data,$params) = $self 
-          ? $self->_args( @_ ) 
-          : Spreadsheet::HTML::_args( @_ );
-
-      # pull out custom named parameters from $params
-      my $color = $params->{color};
-
-      my @params = (
-          # add custom params that client CAN overide here
-          @_,
-          # add custom params that client can NOT overide here
-      );
-
-      $self ? $self->generate( @params ) : Spreadsheet::HTML::generate( @params );
-  }
-
-It is not pretty, but it keeps the named parameters in line even
-if stray, bare array references are used by the client:
-
-  $table->my_generator( [ 'data here' ], color => 'red' );
-
-A simpler, less flexible form is available if you do not need to
-pull out client params:
-
-  use Spreadsheet::HTML;
-
-  sub my_generator {
-      my $self = shift if ref($_[0]) =~ /^Spreadsheet::HTML/;
-
-      my @params = (
-          # add custom params that client CAN overide here
-          @_,
-          # add custom params that client can NOT overide here
-      );
-
-      $self ? $self->generate( @params ) : Spreadsheet::HTML::generate( @params );
-  }
-
-Plans are in the works to simplify this "API," possibly even
-to provide a real plugin interface.
-
 =head1 AUTHOR
 
 Jeff Anderson, C<< <jeffa at cpan.org> >>
