@@ -15,7 +15,7 @@ sub _javascript {
         join( ',', map "'$_'", @{ $args{colors} } ),
     ;
 
-    return Spreadsheet::HTML::Presets::_html_tmpl( code => $js, %args );
+    return Spreadsheet::HTML::Presets::_js_wrapper( code => $js, %args );
 }
 
 sub _js_tmpl {
